/*
 * pack.c
 * Author: Yun-Ching Lee (ycl+@cmu.edu)
 * Copyright 1998 by Carnegie Mellon University
 * All rights reserved
 */

#include <stdio.h>
#include <stdlib.h>
#include "cmupa.h"
#include "pack.h"
#include "convert.h"

/*
 * pack_cmupa() takes a CMUPA data structure and encodes it to network
 * byte-order character string, ready to be transmitted
 * return values: on successful completion, length of buffer
 * on failure, negative value (ERROR = -1)
 */

long 
pack_cmupa(CMUPA * data, char *buffer)
{
	u_short         h_bitmask = data->h_bitmask;
	long            i = 0;	/* byte counter */
	long            j;	/* block counter */

	i += int32tobuf(data->packet_size, buffer + i);
	i += int32tobuf(data->packet_id, buffer + i);
	i += int16tobuf(data->scenario_id, buffer + i);
	i += int16tobuf(data->source_id, buffer + i);
	i += int16tobuf(data->dest_id, buffer + i);
	i += int32tobuf(data->time, buffer + i);
	if (data->num_commands <= 0 || data->command_block == NULL)
		CLEARBIT(h_bitmask, 0);
	if (data->num_sensors <= 0 || data->sensor_block == NULL)
		CLEARBIT(h_bitmask, 1);
	if (data->num_imageries <= 0 || data->imagery_block == NULL)
		CLEARBIT(h_bitmask, 2);
	if (data->num_targets <= 0 || data->target_block == NULL)
		CLEARBIT(h_bitmask, 3);
	if (data->num_events <= 0 || data->event_block == NULL)
		CLEARBIT(h_bitmask, 4);
	i += int16tobuf(data->h_bitmask, buffer + i);

	/* encode command section */
	if (TESTBIT(h_bitmask, 0)) {
		i += int16tobuf(data->num_commands, buffer + i);
		for (j = 0; j < data->num_commands; j++) {
			i += pack_command(&data->command_block[j], buffer + i);
			if (i > MAX_BUFFER_LENGTH)
				return ERROR;
		}
	}
	/* encode sensor section */
	if (TESTBIT(h_bitmask, 1)) {
		i += int16tobuf(data->num_sensors, buffer + i);
		for (j = 0; j < data->num_sensors; j++) {
			i += pack_sensor(&data->sensor_block[j], buffer + i);
			if (i > MAX_BUFFER_LENGTH)
				return ERROR;
		}
	}
	/* encode imagery section */
	if (TESTBIT(h_bitmask, 2)) {
		i += int16tobuf(data->num_imageries, buffer + i);
		for (j = 0; j < data->num_imageries; j++) {
			i += pack_imagery(&data->imagery_block[j], buffer + i);
			if (i > MAX_BUFFER_LENGTH)
				return ERROR;
		}
	}
	/* encode target section */
	if (TESTBIT(h_bitmask, 3)) {
		i += int16tobuf(data->num_targets, buffer + i);
		for (j = 0; j < data->num_targets; j++) {
			i += pack_target(&data->target_block[j], buffer + i);
			if (i > MAX_BUFFER_LENGTH)
				return ERROR;
		}
	}
	/* encode event section */
	if (TESTBIT(h_bitmask, 4)) {
		i += int16tobuf(data->num_events, buffer + i);
		for (j = 0; j < data->num_events; j++) {
			i += pack_event(&data->event_block[j], buffer + i);
			if (i > MAX_BUFFER_LENGTH)
				return ERROR;
		}
	}
	/* correct packet_size if it's off */
	if ((unsigned) i != data->packet_size) {
		data->packet_size = i;
		int32tobuf(data->packet_size, buffer);
	}
	if (i > MAX_BUFFER_LENGTH)
		return ERROR;
	else
		return i;
}


long 
pack_command(COMMAND_BLOCK * command, char *buffer)
{
	u_long          c_bitmask;
	long            i = 0;
	long            k;

	i += int16tobuf(command->block_size, buffer + i);
	i += int16tobuf(command->command_id, buffer + i);
	i += int32tobuf(command->command_type, buffer + i);
	i += int32tobuf(command->c_bitmask, buffer + i);
	c_bitmask = command->c_bitmask;

	/* encode targets fields */
	if (TESTBIT(c_bitmask, 0)) {
		i += int16tobuf(command->num_targets, buffer + i);
		for (k = 0; k < command->num_targets; k++)
			i += int16tobuf(command->target_id[k], buffer + i);
	}
	/* encode events fields */
	if (TESTBIT(c_bitmask, 1)) {
		i += int16tobuf(command->num_events, buffer + i);
		for (k = 0; k < command->num_events; k++)
			i += int16tobuf(command->event_id[k], buffer + i);
	}
	/* encode bitmasks */
	if (TESTBIT(c_bitmask, 2)) {
		i += int32tobuf(command->packet_bitmask.c_bitmask, buffer + i);
		i += int32tobuf(command->packet_bitmask.s_bitmask, buffer + i);
		i += int32tobuf(command->packet_bitmask.i_bitmask, buffer + i);
		i += int32tobuf(command->packet_bitmask.r_bitmask, buffer + i);
		i += int32tobuf(command->packet_bitmask.e_bitmask, buffer + i);
	}
	/* encode bandwidth */
	if (TESTBIT(c_bitmask, 3)) {
		i += int32tobuf(command->bandwidth, buffer + i);
	}
	printf("command block_size = %d\n", command->block_size);
	printf("command i = %ld\n", i);
	/* correct block_size if it's off */
	command->block_size = i;
	int16tobuf(command->block_size, buffer);
	return i;
}


long 
pack_sensor(SENSOR_BLOCK * sensor, char *buffer)
{
	u_long          s_bitmask;
	long            i = 0;
	long            k;

	i += int16tobuf(sensor->block_size, buffer + i);
	i += int16tobuf(sensor->sensor_id, buffer + i);
	i += int16tobuf(sensor->sensor_type, buffer + i);
	i += int16tobuf(sensor->sensor_status, buffer + i);
	i += int16tobuf(sensor->label_len, buffer + i);
	memcpy(buffer + i, sensor->label, sensor->label_len);
	i += sensor->label_len;
	i += int32tobuf(sensor->s_bitmask, buffer + i);
	s_bitmask = sensor->s_bitmask;

	/* encode LVCS position fields */
	if (TESTBIT(s_bitmask, 0)) {
		i += floattobuf(sensor->lvcs_pos.x, buffer + i);
		i += floattobuf(sensor->lvcs_pos.y, buffer + i);
		i += floattobuf(sensor->lvcs_pos.z, buffer + i);
	}
	/* encode LVCS uncertainty fields */
	if (TESTBIT(s_bitmask, 1)) {
		i += floattobuf(sensor->lvcs_err.x, buffer + i);
		i += floattobuf(sensor->lvcs_err.y, buffer + i);
		i += floattobuf(sensor->lvcs_err.z, buffer + i);
	}
	/* encode LVCS velocity fields */
	if (TESTBIT(s_bitmask, 2)) {
		i += floattobuf(sensor->lvcs_vel.x, buffer + i);
		i += floattobuf(sensor->lvcs_vel.y, buffer + i);
		i += floattobuf(sensor->lvcs_vel.z, buffer + i);
	}
	/* encode GPS position fields */
	if (TESTBIT(s_bitmask, 3)) {
		i += int32tobuf(sensor->gps_pos.lat, buffer + i);
		i += int32tobuf(sensor->gps_pos.lon, buffer + i);
		i += floattobuf(sensor->gps_pos.alt, buffer + i);
	}
	/* encode GPS uncertainty fields */
	if (TESTBIT(s_bitmask, 4)) {
		i += int32tobuf(sensor->gps_err.lat, buffer + i);
		i += int32tobuf(sensor->gps_err.lon, buffer + i);
		i += floattobuf(sensor->gps_err.alt, buffer + i);
	}
	/* encode GPS velocity fields */
	if (TESTBIT(s_bitmask, 5)) {
		i += floattobuf(sensor->gps_vel.lat, buffer + i);
		i += floattobuf(sensor->gps_vel.lon, buffer + i);
		i += floattobuf(sensor->gps_vel.alt, buffer + i);
	}
	/* encode mount orientation fields */
	if (TESTBIT(s_bitmask, 6)) {
		i += int32tobuf(sensor->mount_pos.theta, buffer + i);
		i += int32tobuf(sensor->mount_pos.phi, buffer + i);
		i += int32tobuf(sensor->mount_pos.rho, buffer + i);
	}
	/* encode mount orientation uncertainty fields */
	if (TESTBIT(s_bitmask, 7)) {
		i += int32tobuf(sensor->mount_err.theta, buffer + i);
		i += int32tobuf(sensor->mount_err.phi, buffer + i);
		i += int32tobuf(sensor->mount_err.rho, buffer + i);
	}
	/* encode mount velocity fields */
	if (TESTBIT(s_bitmask, 8)) {
		i += floattobuf(sensor->mount_vel.theta, buffer + i);
		i += floattobuf(sensor->mount_vel.phi, buffer + i);
		i += floattobuf(sensor->mount_vel.rho, buffer + i);
	}
	/* encode image size fields */
	if (TESTBIT(s_bitmask, 9)) {
		i += int16tobuf(sensor->image_size.u, buffer + i);
		i += int16tobuf(sensor->image_size.v, buffer + i);
	}
	/* encode FOV fields */
	if (TESTBIT(s_bitmask, 10)) {
		i += int32tobuf(sensor->fov.h_angle, buffer + i);
		i += int32tobuf(sensor->fov.v_angle, buffer + i);
	}
	/* encode sensor internal params fields */
	if (TESTBIT(s_bitmask, 11)) {
		i += int16tobuf(sensor->sen_pars.iris, buffer + i);
		i += int16tobuf(sensor->sen_pars.focus, buffer + i);
		i += int16tobuf(sensor->sen_pars.u_centre, buffer + i);
		i += int16tobuf(sensor->sen_pars.v_centre, buffer + i);
		i += floattobuf(sensor->sen_pars.distortion, buffer + i);
	}
	/* encode pan/tilt/zoom position fields */
	if (TESTBIT(s_bitmask, 12)) {
		i += int32tobuf(sensor->ptz_pos.pan, buffer + i);
		i += int32tobuf(sensor->ptz_pos.tilt, buffer + i);
		i += int32tobuf(sensor->ptz_pos.twist, buffer + i);
		i += floattobuf(sensor->ptz_pos.zoom, buffer + i);
	}
	/* encode pan/tilt/zoom velocity fields */
	if (TESTBIT(s_bitmask, 13)) {
		i += floattobuf(sensor->ptz_vel.pan, buffer + i);
		i += floattobuf(sensor->ptz_vel.tilt, buffer + i);
		i += floattobuf(sensor->ptz_vel.twist, buffer + i);
		i += floattobuf(sensor->ptz_vel.zoom, buffer + i);
	}
	/* encode pan/tilt/zoom region of interest fields */
	if (TESTBIT(s_bitmask, 14)) {
		i += int16tobuf(sensor->ptz_roi.num_pos, buffer + i);
		for (k = 0; k < sensor->ptz_roi.num_pos; k++) {
			i += int32tobuf(sensor->ptz_roi.ptz_coord[k].pan, buffer + i);
			i += int32tobuf(sensor->ptz_roi.ptz_coord[k].tilt, buffer + i);
			i += int32tobuf(sensor->ptz_roi.ptz_coord[k].twist, buffer + i);
			i += floattobuf(sensor->ptz_roi.ptz_coord[k].zoom, buffer + i);
		}
	}
	/* encode LVCS region of interest fields */
	if (TESTBIT(s_bitmask, 15)) {
		i += int16tobuf(sensor->lvcs_roi.num_pos, buffer + i);
		for (k = 0; k < sensor->lvcs_roi.num_pos; k++) {
			i += floattobuf(sensor->lvcs_roi.lvcs_coord[k].x, buffer + i);
			i += floattobuf(sensor->lvcs_roi.lvcs_coord[k].y, buffer + i);
			i += floattobuf(sensor->lvcs_roi.lvcs_coord[k].z, buffer + i);
		}
	}
	/* encode camera footprlong in ptz */
	if (TESTBIT(s_bitmask, 16)) {
		i += int32tobuf(sensor->ptz_fp.top_left.theta, buffer + i);
		i += int32tobuf(sensor->ptz_fp.top_left.phi, buffer + i);
		i += int32tobuf(sensor->ptz_fp.bottom_right.theta, buffer + i);
		i += int32tobuf(sensor->ptz_fp.bottom_right.phi, buffer + i);
	}
	/* encode camera footprlong in LVCS */
	if (TESTBIT(s_bitmask, 17)) {
		i += int16tobuf(sensor->lvcs_fp.num_pos, buffer + i);
		for (k = 0; k < sensor->lvcs_fp.num_pos; k++) {
			i += floattobuf(sensor->lvcs_fp.lvcs_coord[k].x, buffer + i);
			i += floattobuf(sensor->lvcs_fp.lvcs_coord[k].y, buffer + i);
			i += floattobuf(sensor->lvcs_fp.lvcs_coord[k].z, buffer + i);
		}
	}
	/* encode camera limit in ptz format */
	if (TESTBIT(s_bitmask, 18)) {
		i += int32tobuf(sensor->ptz_lim.top_left.theta, buffer + i);
		i += int32tobuf(sensor->ptz_lim.top_left.phi, buffer + i);
		i += int32tobuf(sensor->ptz_lim.bottom_right.theta, buffer + i);
		i += int32tobuf(sensor->ptz_lim.bottom_right.phi, buffer + i);
	}
	/* encode camera limit in LVCS format */
	if (TESTBIT(s_bitmask, 19)) {
		i += int16tobuf(sensor->lvcs_lim.num_pos, buffer + i);
		for (k = 0; k < sensor->lvcs_lim.num_pos; k++) {
			i += floattobuf(sensor->lvcs_lim.lvcs_coord[k].x, buffer + i);
			i += floattobuf(sensor->lvcs_lim.lvcs_coord[k].y, buffer + i);
			i += floattobuf(sensor->lvcs_lim.lvcs_coord[k].z, buffer + i);
		}
	}
	printf("sensor block_size = %d\n", sensor->block_size);
	printf("sensor i = %ld\n", i);
	/* correct block_size if it's off */
	sensor->block_size = i;
	int16tobuf(sensor->block_size, buffer);
	return i;
}

long 
pack_imagery(IMAGERY_BLOCK * imagery, char *buffer)
{
	u_long          i_bitmask;
	long            i = 0;

	i += int32tobuf(imagery->block_size, buffer + i);
	i += int16tobuf(imagery->imagery_id, buffer + i);
	i += int16tobuf(imagery->imagery_type, buffer + i);
	i += int16tobuf(imagery->label_len, buffer + i);
	memcpy(buffer + i, imagery->label, imagery->label_len);
	i += imagery->label_len;
	i += int32tobuf(imagery->i_bitmask, buffer + i);
	i_bitmask = imagery->i_bitmask;
	/* encode image data */
	if (TESTBIT(i_bitmask, 0)) {
		i += int32tobuf(imagery->imagery.time, buffer + i);
		i += int16tobuf(imagery->imagery.width, buffer + i);
		i += int16tobuf(imagery->imagery.height, buffer + i);
		i += int16tobuf(imagery->imagery.format, buffer + i);
		i += int32tobuf(imagery->imagery.size, buffer + i);
		memcpy(buffer + i, imagery->imagery.data, imagery->imagery.size);
		i += imagery->imagery.size;
	}
	/* encode video data */
	if (TESTBIT(i_bitmask, 1)) {
		i += int32tobuf(imagery->video.start_time, buffer + i);
		i += int32tobuf(imagery->video.end_time, buffer + i);
		i += int16tobuf(imagery->video.width, buffer + i);
		i += int16tobuf(imagery->video.height, buffer + i);
		i += int16tobuf(imagery->video.format, buffer + i);
		i += int32tobuf(imagery->video.size, buffer + i);
		memcpy(buffer + i, imagery->imagery.data, imagery->video.size);
		i += imagery->video.size;
	}
	printf("imagery block_size = %ld\n", imagery->block_size);
	printf("imagery i = %ld\n", i);
	/* correct block_size if it's off */
	imagery->block_size = i;
	int32tobuf(imagery->block_size, buffer);
	return i;
}

long 
pack_target(TARGET_BLOCK * target, char *buffer)
{
	u_long          r_bitmask;
	u_long          t_bitmask;
	long            traj_len;
	long            i = 0;
	long            k;
	long            l;
	long            m;

	i += int16tobuf(target->block_size, buffer + i);
	i += int16tobuf(target->target_id, buffer + i);
	i += int16tobuf(target->target_type, buffer + i);
	i += int16tobuf(target->label_len, buffer + i);
	memcpy(buffer + i, target->label, target->label_len);
	i += target->label_len;
	i += int32tobuf(target->r_bitmask, buffer + i);
	r_bitmask = target->r_bitmask;
	i += int32tobuf(target->t_bitmask, buffer + i);
	t_bitmask = target->t_bitmask;

	/* encode target position in img coords */
	if (TESTBIT(r_bitmask, 0)) {
		i += int16tobuf(target->img_pos.u, buffer + i);
		i += int16tobuf(target->img_pos.v, buffer + i);
	}
	/* encode target position in LVCS coords */
	if (TESTBIT(r_bitmask, 1)) {
		i += floattobuf(target->lvcs_pos.x, buffer + i);
		i += floattobuf(target->lvcs_pos.y, buffer + i);
		i += floattobuf(target->lvcs_pos.z, buffer + i);
	}
	/* encode LVCS target uncertainty (covariance) */
	if (TESTBIT(r_bitmask, 2)) {
		for (k = 0; k < 3; k++)
			for (l = 0; l < 3; l++)
				i += floattobuf(target->lvcs_err.C[k][l], buffer + i);
	}
	/* encode target velocity in image coords */
	if (TESTBIT(r_bitmask, 3)) {
		i += floattobuf(target->img_vel.u, buffer + i);
		i += floattobuf(target->img_vel.v, buffer + i);
	}
	/* encode target velocity in LVCS coords */
	if (TESTBIT(r_bitmask, 4)) {
		i += floattobuf(target->lvcs_vel.x, buffer + i);
		i += floattobuf(target->lvcs_vel.y, buffer + i);
		i += floattobuf(target->lvcs_vel.z, buffer + i);
	}
	/* encode target bounding box */
	if (TESTBIT(r_bitmask, 5)) {
		i += int16tobuf(target->bounding_box.top_left.u, buffer + i);
		i += int16tobuf(target->bounding_box.top_left.v, buffer + i);
		i += int16tobuf(target->bounding_box.bottom_right.u, buffer + i);
		i += int16tobuf(target->bounding_box.bottom_right.v, buffer + i);
	}
	/* encode target skeleton */
	if (TESTBIT(r_bitmask, 6)) {
		i += int16tobuf(target->skeleton.centroid.u, buffer + i);
		i += int16tobuf(target->skeleton.centroid.v, buffer + i);
		i += int16tobuf(target->skeleton.length, buffer + i);
		for (k = 0; k < target->skeleton.length; k++) {
			i += int16tobuf(target->skeleton.extremal[k].u, buffer + i);
			i += int16tobuf(target->skeleton.extremal[k].v, buffer + i);
		}
	}
	/* encode target bundary */
	if (TESTBIT(r_bitmask, 7)) {
		i += int16tobuf(target->boundary.length, buffer + i);
		for (k = 0; k < target->boundary.length; k++) {
			i += int16tobuf(target->boundary.value[k].u, buffer + i);
			i += int16tobuf(target->boundary.value[k].v, buffer + i);
		}
	}
	/* encode target size */
	if (TESTBIT(r_bitmask, 8))
		i += int16tobuf(target->target_size, buffer + i);

	/* encode target color histogram */
	if (TESTBIT(r_bitmask, 9)) {
		i += int16tobuf(target->histogram.colour_space, buffer + i);
		i += int16tobuf(target->histogram.channel_1_bins, buffer + i);
		i += int16tobuf(target->histogram.channel_2_bins, buffer + i);
		i += int16tobuf(target->histogram.channel_3_bins, buffer + i);
		for (k = 0; k < target->histogram.channel_1_bins; k++)
			i += int16tobuf(target->histogram.channel_1[k], buffer + i);
		for (k = 0; k < target->histogram.channel_2_bins; k++)
			i += int16tobuf(target->histogram.channel_2[k], buffer + i);
		for (k = 0; k < target->histogram.channel_3_bins; k++)
			i += int16tobuf(target->histogram.channel_3[k], buffer + i);
	}
	/* encode image template */
	if (TESTBIT(r_bitmask, 10)) {
		i += int16tobuf(target->temp.size.u, buffer + i);
		i += int16tobuf(target->temp.size.v, buffer + i);
		i += int16tobuf(target->temp.packing, buffer + i);
		memcpy(buffer + i, target->temp.data, target->temp.size.u * target->temp.size.v);
		i += target->temp.size.u * target->temp.size.v;
	}
	/* encode image depth map */
	if (TESTBIT(r_bitmask, 11)) {
		i += int16tobuf(target->depth_map.size.u, buffer + i);
		i += int16tobuf(target->depth_map.size.v, buffer + i);
		i += int16tobuf(target->depth_map.packing, buffer + i);
		memcpy(buffer + i, target->depth_map.data, target->depth_map.size.u * target->depth_map.size.v);
		i += target->depth_map.size.u * target->depth_map.size.v;
	}
	/* encode depth map resolution table */
	if (TESTBIT(r_bitmask, 12)) {
		i += int16tobuf(target->depth_res.disparities, buffer + i);
		for (k = 0; k < target->depth_res.disparities; k++)
			i += floattobuf(target->depth_res.res_table[k], buffer + i);
	}
	/* encode target behaviour */
	if (TESTBIT(r_bitmask, 13))
		i += int16tobuf(target->behav, buffer + i);

	/* encode trajectory on/off length/timestamp fields */
	if (TESTBIT(t_bitmask, 0)) {
		i += int16tobuf(target->on_off.trajectory_len, buffer + i);
		traj_len = target->on_off.trajectory_len;
		for (k = 0; k < traj_len; k++)
			i += int32tobuf(target->on_off.time[k], buffer + i);

		/* encode trajectory in img coords */
		if (TESTBIT(t_bitmask, 1))
			for (k = 0; k < traj_len; k++) {
				i += int16tobuf(target->traj_img_pos[k].u, buffer + i);
				i += int16tobuf(target->traj_img_pos[k].v, buffer + i);
			}

		/* encode trajectory in LVCS coords */
		if (TESTBIT(t_bitmask, 2))
			for (k = 0; k < traj_len; k++)
				for (l = 0; l < 3; l++)
					for (m = 0; m < 3; m++)
						i += floattobuf(target->traj_lvcs_err[k].C[l][m], buffer + i);

		/* encode trajectory covariance matrix */
		if (TESTBIT(t_bitmask, 2))
			for (k = 0; k < traj_len; k++) {
				i += floattobuf(target->traj_lvcs_pos[k].x, buffer + i);
				i += floattobuf(target->traj_lvcs_pos[k].y, buffer + i);
				i += floattobuf(target->traj_lvcs_pos[k].z, buffer + i);
			}

		/* encode trajectory of bounding box */
		if (TESTBIT(t_bitmask, 4))
			for (k = 0; k < traj_len; k++) {
				i += int16tobuf(target->traj_bound_box[k].top_left.u, buffer + i);
				i += int16tobuf(target->traj_bound_box[k].top_left.v, buffer + i);
				i += int16tobuf(target->traj_bound_box[k].bottom_right.u, buffer + i);
				i += int16tobuf(target->traj_bound_box[k].bottom_right.v, buffer + i);
			}

		/* encode trajectory of skeleton */
		if (TESTBIT(t_bitmask, 5))
			for (k = 0; k < traj_len; k++) {
				i += int16tobuf(target->traj_skeleton[k].centroid.u, buffer + i);
				i += int16tobuf(target->traj_skeleton[k].centroid.v, buffer + i);
				i += int16tobuf(target->traj_skeleton[k].length, buffer + i);
				for (l = 0; l < target->traj_skeleton[k].length; l++) {
					i += int16tobuf(target->traj_skeleton[k].extremal[l].u, buffer + i);
					i += int16tobuf(target->traj_skeleton[k].extremal[l].v, buffer + i);
				}
			}

		/* encode trajectory of boundary */
		if (TESTBIT(t_bitmask, 6))
			for (k = 0; k < traj_len; k++) {
				i += int16tobuf(target->traj_boundary[k].length, buffer + i);
				for (l = 0; l < target->traj_boundary[k].length; l++) {
					i += int16tobuf(target->traj_boundary[k].value[l].u, buffer + i);
					i += int16tobuf(target->traj_boundary[k].value[l].v, buffer + i);
				}
			}

		/* encode trajectory of target size */
		if (TESTBIT(t_bitmask, 7))
			for (k = 0; k < traj_len; k++)
				i += int16tobuf(target->traj_size[k], buffer + i);

		/* encode trajectory of color histogram */
		if (TESTBIT(t_bitmask, 8))
			for (k = 0; k < traj_len; k++) {
				i += int16tobuf(target->traj_histogram[k].colour_space, buffer + i);
				i += int16tobuf(target->traj_histogram[k].channel_1_bins, buffer + i);
				i += int16tobuf(target->traj_histogram[k].channel_2_bins, buffer + i);
				i += int16tobuf(target->traj_histogram[k].channel_3_bins, buffer + i);
				for (l = 0; l < target->traj_histogram[k].channel_1_bins; l++)
					i += int16tobuf(target->traj_histogram[k].channel_1[l], buffer + i);
				for (l = 0; l < target->traj_histogram[k].channel_2_bins; l++)
					i += int16tobuf(target->traj_histogram[k].channel_2[l], buffer + i);
				for (l = 0; l < target->traj_histogram[k].channel_3_bins; l++)
					i += int16tobuf(target->traj_histogram[k].channel_3[l], buffer + i);
			}

		/* encode trajectory of image template */
		if (TESTBIT(t_bitmask, 9))
			for (k = 0; k < traj_len; k++) {
				i += int16tobuf(target->traj_template[k].size.u, buffer + i);
				i += int16tobuf(target->traj_template[k].size.v, buffer + i);
				i += int16tobuf(target->traj_template[k].packing, buffer + i);
				memcpy(buffer + i, target->traj_template[k].data, target->traj_template[k].size.u * target->traj_template[k].size.v);
				i += target->traj_template[k].size.u * target->traj_template[k].size.v;
			}

		/* encode trajectory depth map */
		if (TESTBIT(t_bitmask, 10))
			for (k = 0; k < traj_len; k++) {
				i += int16tobuf(target->traj_depth_map[k].size.u, buffer + i);
				i += int16tobuf(target->traj_depth_map[k].size.v, buffer + i);
				i += int16tobuf(target->traj_depth_map[k].packing, buffer + i);
				memcpy(buffer + i, target->traj_depth_map[k].data, target->traj_depth_map[k].size.u * target->traj_depth_map[k].size.v);
				i += target->traj_depth_map[k].size.u * target->traj_depth_map[k].size.v;
			}

		/* encode trajectory behaviour */
		if (TESTBIT(t_bitmask, 11))
			for (k = 0; k < traj_len; k++)
				i += int16tobuf(target->traj_behav[k], buffer + i);
	}
	printf("target block_size = %d\n", target->block_size);
	printf("target i = %ld\n", i);
	/* correct block_size if it's off */
	target->block_size = i;
	int16tobuf(target->block_size, buffer);
	return i;
}

long 
pack_event(EVENT_BLOCK * event, char *buffer)
{
	u_long          e_bitmask;
	long            i = 0;
	long            k;

	i += int16tobuf(event->block_size, buffer + i);
	i += int16tobuf(event->event_id, buffer + i);
	i += int16tobuf(event->event_type, buffer + i);
	i += int16tobuf(event->label_len, buffer + i);
	memcpy(buffer + i, event->label, event->label_len);
	i += event->label_len;
	i += int32tobuf(event->e_bitmask, buffer + i);
	e_bitmask = event->e_bitmask;

	/* encode event position in img coords */
	if (TESTBIT(e_bitmask, 0)) {
		i += int16tobuf(event->event_img_pos.u, buffer + i);
		i += int16tobuf(event->event_img_pos.v, buffer + i);
		i += floattobuf(event->event_img_range, buffer + i);
		i += int32tobuf(event->event_img_time, buffer + i);
	}
	/* encode event position in LVCS coords */
	if (TESTBIT(e_bitmask, 1)) {
		i += floattobuf(event->event_lvcs_pos.x, buffer + i);
		i += floattobuf(event->event_lvcs_pos.y, buffer + i);
		i += floattobuf(event->event_lvcs_pos.z, buffer + i);
		i += int32tobuf(event->event_lvcs_time, buffer + i);
	}
	/* encode target list */
	if (TESTBIT(e_bitmask, 2)) {
		i += int16tobuf(event->num_targets, buffer + i);
		for (k = 0; k < event->num_targets; k++)
			i += int16tobuf(event->target_id[k], buffer + i);
	}
	printf("event block_size = %d\n", event->block_size);
	printf("event i = %ld\n", i);
	/* correct block_size if it's off */
	event->block_size = i;
	int16tobuf(event->block_size, buffer);
	return i;
}
